@@ -1,4 +1,3 @@
-import asyncio
 import os
 import warnings
 from functools import partial
@@ -82,11 +81,9 @@ class CLIPEncoder(Executor):
 
     @requests(on='/rank')
     async def rank(self, docs: 'DocumentArray', parameters: Dict, **kwargs):
-        _source = parameters.get('source', '@m')
+        await self.encode(docs['@r,m'])
 
-        await asyncio.gather(self.encode(docs), self.encode(docs[_source]))
-
-        set_rank(docs, _source)
+        set_rank(docs)
 
     @requests
     async def encode(self, docs: 'DocumentArray', **kwargs):
@@ -97,7 +94,7 @@ class CLIPEncoder(Executor):
 
         # for image
         if _img_da:
-            for minibatch in _img_da.map_batch(
+            for minibatch, _contents in _img_da.map_batch(
                 partial(
                     preproc_image, preprocess_fn=self._preprocess_tensor, return_np=True
                 ),
@@ -105,16 +102,23 @@ class CLIPEncoder(Executor):
                 pool=self._pool,
             ):
                 minibatch.embeddings = self._model.encode_image(minibatch.tensors)
+                # recover original content
+                if _contents:
+                    for _d, _ct in zip(minibatch, _contents):
+                        _d.content = _ct
 
         # for text
         if _txt_da:
-            for minibatch, _texts in _txt_da.map_batch(
+            for minibatch, _contents in _txt_da.map_batch(
                 partial(preproc_text, return_np=True),
                 batch_size=self._minibatch_size,
                 pool=self._pool,
             ):
                 minibatch.embeddings = self._model.encode_text(minibatch.tensors)
-                minibatch.texts = _texts
+                # recover original content
+                if _contents:
+                    for _d, _ct in zip(minibatch, _contents):
+                        _d.content = _ct
 
         # drop tensors
         docs.tensors = None
